@@ -68,8 +68,7 @@ GRAPHENE_BEGIN_DECLS
  *
  * Since: 1.0
  */
-GRAPHENE_VECTORCALL
-static inline graphene_simd4x4f_t
+static inline graphene_simd4x4f_t GRAPHENE_VECTORCALL
 graphene_simd4x4f_init (graphene_simd4f_t x,
                         graphene_simd4f_t y,
                         graphene_simd4f_t z,
@@ -425,19 +424,14 @@ graphene_simd4x4f_matrix_mul (const graphene_simd4x4f_t *a,
    * better numbers while retaining the same correct results as above:
    * the scalar implementation now clocks at 91ns; the GCC vector
    * implementation is 19ns; and the SSE implementation is 16ns.
-   */
-  const graphene_simd4f_t row1 = a->x;
-  const graphene_simd4f_t row2 = a->y;
-  const graphene_simd4f_t row3 = a->z;
-  const graphene_simd4f_t row4 = a->w;
-
-  /* the order is correct if we want to multiply A with B; remember
+   *
+   * the order is correct if we want to multiply A with B; remember
    * that matrix multiplication is non-commutative.
    */
-  graphene_simd4x4f_vec4_mul (b, &row1, &res->x);
-  graphene_simd4x4f_vec4_mul (b, &row2, &res->y);
-  graphene_simd4x4f_vec4_mul (b, &row3, &res->z);
-  graphene_simd4x4f_vec4_mul (b, &row4, &res->w);
+  graphene_simd4x4f_vec4_mul (b, &a->x, &res->x);
+  graphene_simd4x4f_vec4_mul (b, &a->y, &res->y);
+  graphene_simd4x4f_vec4_mul (b, &a->z, &res->z);
+  graphene_simd4x4f_vec4_mul (b, &a->w, &res->w);
 #endif
 }
 
@@ -518,7 +512,7 @@ graphene_simd4x4f_init_ortho (graphene_simd4x4f_t *m,
  * graphene_simd4x4f_init_look_at:
  * @m: a #graphene_simd4x4f_t
  * @eye: vector for the camera coordinates
- * @center: vector the the object coordinates
+ * @center: vector for the object coordinates
  * @up: vector for the upwards direction
  *
  * Initializes a SIMD matrix with the projection necessary for
@@ -829,9 +823,11 @@ graphene_simd4x4f_div (const graphene_simd4x4f_t *a,
  *
  * Inverts the given #graphene_simd4x4f_t.
  *
+ * Returns: `true` if the matrix was invertible
+ *
  * Since: 1.0
  */
-static inline void
+static inline bool
 graphene_simd4x4f_inverse (const graphene_simd4x4f_t *m,
                            graphene_simd4x4f_t       *res)
 {
@@ -901,17 +897,24 @@ graphene_simd4x4f_inverse (const graphene_simd4x4f_t *m,
   const graphene_simd4f_t d0 = graphene_simd4f_mul (r1_sum, r0);
   const graphene_simd4f_t d1 = graphene_simd4f_add (d0, graphene_simd4f_merge_high (d0, d0));
   const graphene_simd4f_t det = graphene_simd4f_sub (d1, graphene_simd4f_splat_y (d1));
-  const graphene_simd4f_t invdet = graphene_simd4f_splat_x (graphene_simd4f_div (graphene_simd4f_splat (1.0f), det));
+  if (graphene_simd4f_get_x (det) != 0.f)
+    {
+      const graphene_simd4f_t invdet = graphene_simd4f_splat_x (graphene_simd4f_div (graphene_simd4f_splat (1.0f), det));
 
-  const graphene_simd4f_t o0 = graphene_simd4f_mul (graphene_simd4f_flip_sign_0101 (r1_sum), invdet);
-  const graphene_simd4f_t o1 = graphene_simd4f_mul (graphene_simd4f_flip_sign_1010 (r0_sum), invdet);
-  const graphene_simd4f_t o2 = graphene_simd4f_mul (graphene_simd4f_flip_sign_0101 (r3_sum), invdet);
-  const graphene_simd4f_t o3 = graphene_simd4f_mul (graphene_simd4f_flip_sign_1010 (r2_sum), invdet);
+      const graphene_simd4f_t o0 = graphene_simd4f_mul (graphene_simd4f_flip_sign_0101 (r1_sum), invdet);
+      const graphene_simd4f_t o1 = graphene_simd4f_mul (graphene_simd4f_flip_sign_1010 (r0_sum), invdet);
+      const graphene_simd4f_t o2 = graphene_simd4f_mul (graphene_simd4f_flip_sign_0101 (r3_sum), invdet);
+      const graphene_simd4f_t o3 = graphene_simd4f_mul (graphene_simd4f_flip_sign_1010 (r2_sum), invdet);
 
-  graphene_simd4x4f_t mt = graphene_simd4x4f_init (o0, o1, o2, o3);
+      graphene_simd4x4f_t mt = graphene_simd4x4f_init (o0, o1, o2, o3);
 
-  /* transpose the resulting matrix */
-  graphene_simd4x4f_transpose (&mt, res);
+      /* transpose the resulting matrix */
+      graphene_simd4x4f_transpose (&mt, res);
+
+      return true;
+    }
+
+  return false;
 }
 
 /**
